--- LinuxBIOSv2/util/probe_superio/probe_superio.c	(Revision 2744)
+++ LinuxBIOSv2/util/probe_superio/probe_superio.c	(Arbeitskopie)
@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2006 Ronald Minnich <rminnich@gmail.com>
  * Copyright (C) 2006 coresystems GmbH <info@coresystems.de>
+ * Copyright (C) 2007 Carl-Daniel Hailfinger
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,11 +33,16 @@
   * if we need this. 
   */
 
-unsigned char regval(unsigned short port, unsigned short reg) {
+unsigned char regval(unsigned short port, unsigned char reg) {
 	outb(reg, port);
 	return inb(port+1);
 }
 
+void regwrite(unsigned short port, unsigned char reg, unsigned char val) {
+	outb(reg, port);
+	outb(val, port+1);
+}
+
 void
 dump_ns8374(unsigned short port) {
 	printf("Enables: 21=%02x, 22=%02x, 23=%02x, 24=%02x, 26=%02x\n", 
@@ -46,15 +52,13 @@
 	/* check COM1. This is all we care about at present. */
 	printf("COM 1 is Globally %s\n", regval(port,0x26)&8 ? "disabled" : "enabled");
 	/* select com1 */
-	outb(0x7, port);
-	outb(3, port+1);
+	regwrite(port, 0x07, 0x03);
 	printf("COM 1 is locally %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("COM1 60=%02x, 61=%02x, 70=%02x, 71=%02x, 74=%02x, 75=%02x, f0=%02x\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70), regval(port, 0x71),
 		regval(port, 0x74), regval(port, 0x75), regval(port, 0xf0));
 	/* select gpio */
-	outb(0x7, port);
-	outb(7, port+1);
+	regwrite(port, 0x07, 0x07);
 	printf("GPIO is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("GPIO 60=%02x, 61=%02x, 70=%02x, 71=%02x, 74=%02x, 75=%02x, f0=%02x\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70), regval(port, 0x71),
@@ -85,38 +89,33 @@
 	printf("2b=%02x\n", regval(port, 0x2b));
 
 	/* select UART 1 */
-	outb(0x07, port);
-	outb(0x01, port+1);
+	regwrite(port, 0x07, 0x01);
 	printf("UART1 is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("UART1 base=%02x%02x, irq=%02x, mode=%s\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70)&0x0f, 
 		regval(port, 0xf0)&0x10 ? "RS485":"RS232");
 
 	/* select UART 2 */
-	outb(0x07, port);
-	outb(0x02, port+1);
+	regwrite(port, 0x07, 0x02);
 	printf("UART2 is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("UART2 base=%02x%02x, irq=%02x, mode=%s\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70)&0x0f, 
 		regval(port, 0xf0)&0x10 ? "RS485":"RS232");
 
 	/* select Parport */
-	outb(0x07, port);
-	outb(0x03, port+1);
+	regwrite(port, 0x07, 0x03);
 	printf("PARPORT is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("PARPORT base=%02x%02x, irq=%02x\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70)&0x0f);
 
 	/* select hw monitor */
-	outb(0x07, port);
-	outb(0x04, port+1);
+	regwrite(port, 0x07, 0x04);
 	printf("HW monitor is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("HW monitor base=%02x%02x, irq=%02x\n", 
 		regval(port, 0x60), regval(port, 0x61), regval(port, 0x70)&0x0f);
 
 	/* select gpio */
-	outb(0x07, port);
-	outb(0x06, port+1);
+	regwrite(port, 0x07, 0x05);
 	printf("GPIO is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
 	printf("GPIO 70=%02x, e0=%02x, e1=%02x, e2=%02x, e3=%02x, e4=%02x, e5=%02x\n", 
 		regval(port, 0x70), regval(port, 0xe0), regval(port, 0xe1), regval(port, 0xe2),
@@ -130,7 +129,30 @@
 
 }
 
+void
+dump_ite(unsigned short port, unsigned short id) 
+{
+	int i;
+	printf ("ITE chip\n");
 
+	for (i=0x20; i<=0x24; i++)
+		printf("index %02x=%02x\n", i, regval(port, i));
+
+	switch(id) {
+		case 0x8702:
+		case 0x8705:
+		case 0x8710:
+		case 0x8712:
+		case 0x8716:
+		case 0x8718:
+		default:
+			break;
+	}
+	/* select UART 1 */
+	regwrite(port, 0x07, 0x01);
+	printf("UART1 is %s\n", regval(port, 0x30) & 1 ? "enabled" : "disabled");
+}
+
 void
 probe_idregs_simple(unsigned short port){
 	unsigned char id;
@@ -167,9 +189,10 @@
 
 void
 probe_idregs_fintek(unsigned short port){
-	unsigned int vid, did;
+	unsigned int vid, did, success = 0;
 
 	// Enable configuration sequence (Fintek uses this for example)
+	// Older ITE chips have the same enable sequence
 	outb(0x87, port);
 	outb(0x87, port);
 
@@ -185,13 +208,10 @@
 	}
 	did = inb(port+1);
 
-	outb(0x21, port);
-	did = did|(inb(port+1)<<8);
+	did = did|(regval(port, 0x21)<<8);
 
-	outb(0x23, port);
-	vid = inb(port+1);
-	outb(0x24, port);
-	vid = vid|(inb(port+1)<<8);
+	vid = regval(port, 0x23);
+	vid = vid|(regval(port, 0x24)<<8);
 
 	printf("SuperIO found at 0x%02x: vid=0x%04x/did=0x%04x\n", port, vid, did);
 
@@ -199,23 +219,103 @@
 		return;
 
 	// printf("%s\n", familyid[id]);
+	switch(did) {
+		case 0x1087: // reversed for ITE8710
+			success = 1;
+			dump_ite(port, ((did & 0xff) << 8) | ((did & 0xff00) >> 8));
+			// disable configuration
+			regwrite(port, 0x02, 0x02);
+			break;
+		default:
+			break;
+	}
 	switch(vid) {
 		case 0x3419:
+			success = 1;
 			dump_fintek(port, did);
 			break;
 		default:
-			printf("no dump for 0x%04x/0x%04x\n", vid, did);
 			break;
 	}
+	if (!success)
+		printf("no dump for vid 0x%04x, did 0x%04x\n", vid, did);
 
-	// disable configuration
+	// disable configuration (for Fintek, doesn't hurt ITE)
 	outb(0xaa, port);
 }
 
 void
+probe_idregs_ite(unsigned short port){
+	unsigned int id, chipver;
+
+	// Enable configuration sequence (ITE uses this for newer IT87[012]x)
+	// IT871[01] uses 0x87, 0x87 -> fintek detection should handle it
+	// IT8761 uses 0x87, 0x61, 0x55, 0x55/0xaa
+	// IT86xx series uses different ports
+	// IT8661 uses 0x86, 0x61, 0x55/0xaa, 0x55/0xaa and 32 more writes
+	// IT8673 uses 0x86, 0x80, 0x55/0xaa, 0x55/0xaa and 32 more writes
+	outb(0x87, port);
+	outb(0x01, port);
+	outb(0x55, port);
+	if (port == 0x2e)
+		outb(0x55, port);
+	else
+		outb(0xAA, port);
+
+	// Read Chip ID Byte 1
+	id = regval(port, 0x20);
+	if (id != 0x87)  {
+		if (inb(port) == 0xff )
+			printf ("No SuperIO chip found at 0x%04x\n", port);
+		else
+			printf("probing 0x%04x, failed (0x%02x), data returns 0x%02x\n", 
+					port, inb(port), inb(port+1));
+		return;
+	}
+
+	id <<= 8;
+
+	// Read Chip ID Byte 2
+	id |= regval(port, 0x21);
+
+	// Read Chip Version, only bit 3..0 for all IT87xx
+	chipver = regval(port, 0x22) & 0x0f;
+
+	/* ID Mapping Table
+	   unknown -> IT8711 (no datasheet)
+	   unknown -> IT8722 (no datasheet)
+	   0x8702 -> IT8702
+	   0x8705 -> IT8700 or IT8705
+	   0x8710 -> IT8710
+	   0x8712 -> IT8712
+	   0x8716 -> IT8716 or IT8726 (identical except CPU voltage control)
+	   0x8718 -> IT8718
+	*/
+	printf("ITE? SuperIO found at 0x%02x: id=0x%04x, chipver=0x%01x\n",
+	       port, id, chipver);
+
+	switch(id) {
+		case 0x8702:
+		case 0x8705:
+		case 0x8710: //pointless, IT8710 has different enable sequence
+		case 0x8712:
+		case 0x8716:
+		case 0x8718:
+			dump_ite(port, id);
+			break;
+		default:
+			printf("no dump for id 0x%04x\n", id);
+			break;
+	}
+	// disable configuration
+	regwrite(port, 0x02, 0x02);
+}
+
+void
 probe_superio(unsigned short port) {
 	probe_idregs_simple(port);
 	probe_idregs_fintek(port);
+	probe_idregs_ite(port);
 }
 
 int
